@@ -63,6 +63,8 @@ pub fn try_from_factor(factor: Factor) -> Result<Symbol> {
         Factor::NonTerminal(n) => Ok(Symbol::n(&n)),
         Factor::Terminal(t, s) => Ok(Symbol::t(&t, s)),
         Factor::ScannerSwitch(s) => Ok(Symbol::s(s)),
+        Factor::ScannerSwitchPush(s) => Ok(Symbol::Push(s)),
+        Factor::ScannerSwitchPop => Ok(Symbol::Pop),
         _ => Err(format!("Unexpected type of factor: {}", factor).into()),
     }
 }
